package filicop

import (
	"database/sql"
	"log"
	"os"
	"time"

	_ "github.com/go-sql-driver/mysql"
)

// db stores the connection to the database
var db *sql.DB

// DSNGenerator returns DSN filed with values from environment
func DSNGenerator() string {
	db_user := "filicop"
	db_password := "filicop"
	db_host := ""
	db_db := "filicop"

	if v, exists := os.LookupEnv("MYSQL_HOST"); exists {
		db_host = v
	}
	if v, exists := os.LookupEnv("MYSQL_PASSWORD"); exists {
		db_password = v
	} else if v, exists := os.LookupEnv("MYSQL_ROOT_PASSWORD"); exists {
		db_user = "root"
		db_password = v
	}
	if v, exists := os.LookupEnv("MYSQL_USER"); exists {
		db_user = v
	}
	if v, exists := os.LookupEnv("MYSQL_DATABASE"); exists {
		db_db = v
	}

	return db_user + ":" + db_password + "@" + db_host + "/" + db_db
}

// DBInit establishes the connection to the database
func DBInit(dsn string) (err error) {
	if db, err = sql.Open("mysql", dsn+"?parseTime=true&foreign_key_checks=1"); err != nil {
		return
	}

	_, err = db.Exec(`SET SESSION sql_mode = 'STRICT_TRANS_TABLES,NO_ZERO_IN_DATE,NO_ZERO_DATE,ERROR_FOR_DIVISION_BY_ZERO';`)
	for i := 0; err != nil && i < 15; i += 1 {
		if _, err = db.Exec(`SET SESSION sql_mode = 'STRICT_TRANS_TABLES,NO_ZERO_IN_DATE,NO_ZERO_DATE,ERROR_FOR_DIVISION_BY_ZERO';`); err != nil && i <= 5 {
			log.Println("An error occurs when trying to connect to DB, will retry in 2 seconds: ", err)
			time.Sleep(2 * time.Second)
		}
	}

	return
}

// DBCreate creates all necessary tables used by the package
func DBCreate() error {
	if _, err := db.Exec(`
CREATE TABLE IF NOT EXISTS fleets(
  id_fleet INTEGER NOT NULL PRIMARY KEY AUTO_INCREMENT,
  label VARCHAR(255) NOT NULL
) DEFAULT CHARACTER SET = utf8 COLLATE = utf8_bin;
`); err != nil {
		return err
	}
	if _, err := db.Exec(`
CREATE TABLE IF NOT EXISTS vehicles(
  id_vehicle INTEGER NOT NULL PRIMARY KEY AUTO_INCREMENT,
  id_fleet INTEGER NOT NULL,
  label VARCHAR(255) NOT NULL,
  FOREIGN KEY(id_fleet) REFERENCES fleets(id_fleet)
) DEFAULT CHARACTER SET = utf8 COLLATE = utf8_bin;
`); err != nil {
		return err
	}
	if _, err := db.Exec(`
CREATE TABLE IF NOT EXISTS vehicle_locations(
  id_location INTEGER NOT NULL PRIMARY KEY AUTO_INCREMENT,
  id_vehicle INTEGER NOT NULL,
  date DATETIME NOT NULL,
  location POINT NOT NULL,
  FOREIGN KEY(id_vehicle) REFERENCES vehicles(id_vehicle)
) DEFAULT CHARACTER SET = utf8 COLLATE = utf8_bin;
`); err != nil {
		return err
	}
	if _, err := db.Exec(`
CREATE TABLE IF NOT EXISTS requestors(
  id_requestor INTEGER NOT NULL PRIMARY KEY AUTO_INCREMENT,
  id_fleet INTEGER NOT NULL,
  name VARCHAR(255) NOT NULL,
  FOREIGN KEY(id_fleet) REFERENCES fleets(id_fleet)
) DEFAULT CHARACTER SET = utf8 COLLATE = utf8_bin;
`); err != nil {
		return err
	}
	if _, err := db.Exec(`
CREATE TABLE IF NOT EXISTS missions(
  id_mission INTEGER NOT NULL PRIMARY KEY AUTO_INCREMENT,
  id_requestor INTEGER NOT NULL,
  id_vehicle INTEGER,
  creation DATETIME NOT NULL,
  location_source POINT NOT NULL,
  location_destination POINT NOT NULL,
  FOREIGN KEY(id_vehicle) REFERENCES vehicles(id_vehicle),
  FOREIGN KEY(id_requestor) REFERENCES requestors(id_requestor)
) DEFAULT CHARACTER SET = utf8 COLLATE = utf8_bin;
`); err != nil {
		return err
	}
	return nil
}

// DBClose closes the connection to the database
func DBClose() error {
	return db.Close()
}

func DBPrepare(query string) (*sql.Stmt, error) {
	return db.Prepare(query)
}

func DBQuery(query string, args ...interface{}) (*sql.Rows, error) {
	return db.Query(query, args...)
}

func DBExec(query string, args ...interface{}) (sql.Result, error) {
	return db.Exec(query, args...)
}

func DBQueryRow(query string, args ...interface{}) *sql.Row {
	return db.QueryRow(query, args...)
}
